@@ -41,41 +41,68 @@
 #include "load-graph.h"
 #include "util.h"
 #include "disks.h"
+#include "settings-keys.h"
 #include "gsm_color_button.h"
 
 
+static gboolean
+cb_window_key_press_event (GtkWidget *widget, GdkEvent *event, gpointer user_data)
+{
+    const char *current_page = gtk_stack_get_visible_child_name (GTK_STACK (GsmApplication::get()->stack));
+
+    if (strcmp (current_page, "processes") == 0)
+        return gtk_search_bar_handle_event (GTK_SEARCH_BAR (user_data), event);
+    
+    return FALSE;
+}
+
+static void
+search_text_changed (GtkEditable *entry, gpointer data)
+{
+    GsmApplication * const app = static_cast<GsmApplication *>(data);
+    gtk_tree_model_filter_refilter (GTK_TREE_MODEL_FILTER (gtk_tree_model_sort_get_model (
+                                    GTK_TREE_MODEL_SORT (gtk_tree_view_get_model(
+                                    GTK_TREE_VIEW (app->tree))))));
+}
+
 static void 
-create_proc_view(ProcmanApp *app, GtkBuilder * builder)
+create_proc_view(GsmApplication *app, GtkBuilder * builder)
 {
     GtkWidget *proctree;
     GtkWidget *scrolled;
-    char* string;
-
-    /* create the processes tab */
-    string = make_loadavg_string ();
-    app->loadavg = GTK_WIDGET (gtk_builder_get_object (builder, "load_avg_label"));
-    gtk_label_set_text (GTK_LABEL (app->loadavg), string);
-    g_free (string);
 
     proctree = proctable_new (app);
     scrolled = GTK_WIDGET (gtk_builder_get_object (builder, "processes_scrolled"));
 
     gtk_container_add (GTK_CONTAINER (scrolled), proctree);
 
+    app->proc_toolbar_revealer = GTK_WIDGET (gtk_builder_get_object (builder, "proc_toolbar_revealer"));
+
     /* create popup_menu for the processes tab */
     GMenuModel *menu_model = G_MENU_MODEL (gtk_builder_get_object (builder, "process-popup-menu"));
     app->popup_menu = gtk_menu_new_from_model (menu_model);
     gtk_menu_attach_to_widget (GTK_MENU (app->popup_menu), app->main_window, NULL);
+    
+    GtkSearchBar *search_bar = GTK_SEARCH_BAR (gtk_builder_get_object (builder, "proc_searchbar"));
+    app->search_entry = GTK_WIDGET (gtk_builder_get_object (builder, "proc_searchentry"));
+    
+    gtk_search_bar_connect_entry (search_bar, GTK_ENTRY (app->search_entry));
+    g_signal_connect (app->main_window, "key-press-event",
+                      G_CALLBACK (cb_window_key_press_event), search_bar);
+                  
+    g_signal_connect (app->search_entry, "changed", G_CALLBACK (search_text_changed), app);
+
+    g_object_bind_property (app->search_button, "active", search_bar, "search-mode-enabled", G_BINDING_BIDIRECTIONAL);
 }
 
 void
-cb_cpu_color_changed (GSMColorButton *cp, gpointer data)
+cb_cpu_color_changed (GsmColorButton *cp, gpointer data)
 {
     guint cpu_i = GPOINTER_TO_UINT (data);
     GSettings *settings = g_settings_new (GSM_GSETTINGS_SCHEMA);
 
     /* Get current values */
-    GVariant *cpu_colors_var = g_settings_get_value(settings, "cpu-colors");
+    GVariant *cpu_colors_var = g_settings_get_value (settings, GSM_SETTING_CPU_COLORS);
     gsize children_n = g_variant_n_children(cpu_colors_var);
 
     /* Create builder to contruct new setting with updated value for cpu i */
@@ -97,11 +124,12 @@ cb_cpu_color_changed (GSMColorButton *cp, gpointer data)
     }
 
     /* Just set the value and let the changed::cpu-colors signal callback do the rest. */
-    g_settings_set_value(settings, "cpu-colors", g_variant_builder_end(&builder));
+    g_settings_set_value (settings, GSM_SETTING_CPU_COLORS,
+                          g_variant_builder_end(&builder));
 }
 
 static void change_settings_color(GSettings *settings, const char *key,
-                                  GSMColorButton *cp)
+                                  GsmColorButton *cp)
 {
     GdkRGBA c;
     char *color;
@@ -113,39 +141,38 @@ static void change_settings_color(GSettings *settings, const char *key,
 }
 
 static void
-cb_mem_color_changed (GSMColorButton *cp, gpointer data)
+cb_mem_color_changed (GsmColorButton *cp, gpointer data)
 {
-    ProcmanApp *app = (ProcmanApp *) data;
-    change_settings_color(app->settings, "mem-color", cp);
+    GsmApplication *app = (GsmApplication *) data;
+    change_settings_color (app->settings, GSM_SETTING_MEM_COLOR, cp);
 }
 
 
 static void
-cb_swap_color_changed (GSMColorButton *cp, gpointer data)
+cb_swap_color_changed (GsmColorButton *cp, gpointer data)
 {
-    ProcmanApp *app = (ProcmanApp *) data;
-    change_settings_color(app->settings, "swap-color", cp);
+    GsmApplication *app = (GsmApplication *) data;
+    change_settings_color (app->settings, GSM_SETTING_SWAP_COLOR, cp);
 }
 
 static void
-cb_net_in_color_changed (GSMColorButton *cp, gpointer data)
+cb_net_in_color_changed (GsmColorButton *cp, gpointer data)
 {
-    ProcmanApp *app = (ProcmanApp *) data;
-    change_settings_color(app->settings, "net-in-color", cp);
+    GsmApplication *app = (GsmApplication *) data;
+    change_settings_color (app->settings, GSM_SETTING_NET_IN_COLOR, cp);
 }
 
 static void
-cb_net_out_color_changed (GSMColorButton *cp, gpointer data)
+cb_net_out_color_changed (GsmColorButton *cp, gpointer data)
 {
-    ProcmanApp *app = (ProcmanApp *) data;
-    change_settings_color(app->settings, "net-out-color", cp);
+    GsmApplication *app = (GsmApplication *) data;
+    change_settings_color(app->settings, GSM_SETTING_NET_OUT_COLOR, cp);
 }
 
 static void
-create_sys_view (ProcmanApp *app, GtkBuilder * builder)
+create_sys_view (GsmApplication *app, GtkBuilder * builder)
 {
     GtkWidget *cpu_graph_box, *mem_graph_box, *net_graph_box;
-    GtkWidget *cpu_exp, *mem_exp, *net_exp;
     GtkWidget *label,*cpu_label;
     GtkWidget *table;
     GtkWidget *color_picker;
@@ -177,6 +204,7 @@ create_sys_view (ProcmanApp *app, GtkBuilder * builder)
         GtkWidget *temp_hbox;
 
         temp_hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+        gtk_widget_show (temp_hbox);
         if (i < cols) {
             gtk_grid_insert_column(GTK_GRID(cpu_table), i%cols);
         }
@@ -185,7 +213,7 @@ create_sys_view (ProcmanApp *app, GtkBuilder * builder)
         }
         gtk_grid_attach(GTK_GRID (cpu_table), temp_hbox, i%cols, i/cols, 1, 1);
         color_picker = gsm_color_button_new (&cpu_graph->colors.at(i), GSMCP_TYPE_CPU);
-        g_signal_connect (G_OBJECT (color_picker), "color_set",
+        g_signal_connect (G_OBJECT (color_picker), "color-set",
                           G_CALLBACK (cb_cpu_color_changed), GINT_TO_POINTER (i));
         gtk_box_pack_start (GTK_BOX (temp_hbox), color_picker, FALSE, TRUE, 0);
         gtk_widget_set_size_request(GTK_WIDGET(color_picker), 32, -1);
@@ -199,11 +227,14 @@ create_sys_view (ProcmanApp *app, GtkBuilder * builder)
         gsm_color_button_set_title(GSM_COLOR_BUTTON(color_picker), title_text);
         g_free(title_text);
         gtk_box_pack_start (GTK_BOX (temp_hbox), label, FALSE, FALSE, 6);
+        gtk_widget_show (label);
         g_free (label_text);
 
         cpu_label = gtk_label_new (NULL);
+
         gtk_misc_set_alignment (GTK_MISC (cpu_label), 0.0, 0.5);
         gtk_box_pack_start (GTK_BOX (temp_hbox), cpu_label, FALSE, FALSE, 0);
+        gtk_widget_show (cpu_label);
         load_graph_get_labels(cpu_graph)->cpu[i] = cpu_label;
 
     }
@@ -224,7 +255,7 @@ create_sys_view (ProcmanApp *app, GtkBuilder * builder)
     table = GTK_WIDGET (gtk_builder_get_object (builder, "mem_table"));
 
     color_picker = load_graph_get_mem_color_picker(mem_graph);
-    g_signal_connect (G_OBJECT (color_picker), "color_set",
+    g_signal_connect (G_OBJECT (color_picker), "color-set",
                       G_CALLBACK (cb_mem_color_changed), app);
     title_text = g_strdup_printf(title_template, _("Memory"));
     gsm_color_button_set_title(GSM_COLOR_BUTTON(color_picker), title_text);
@@ -236,7 +267,7 @@ create_sys_view (ProcmanApp *app, GtkBuilder * builder)
     gtk_grid_attach_next_to (GTK_GRID (table), load_graph_get_labels(mem_graph)->memory, label, GTK_POS_BOTTOM, 1, 1);
 
     color_picker = load_graph_get_swap_color_picker(mem_graph);
-    g_signal_connect (G_OBJECT (color_picker), "color_set",
+    g_signal_connect (G_OBJECT (color_picker), "color-set",
                       G_CALLBACK (cb_swap_color_changed), app);
     title_text = g_strdup_printf(title_template, _("Swap"));
     gsm_color_button_set_title(GSM_COLOR_BUTTON(color_picker), title_text);
@@ -264,7 +295,7 @@ create_sys_view (ProcmanApp *app, GtkBuilder * builder)
 
     color_picker = gsm_color_button_new (
         &net_graph->colors.at(0), GSMCP_TYPE_NETWORK_IN);
-    g_signal_connect (G_OBJECT (color_picker), "color_set",
+    g_signal_connect (G_OBJECT (color_picker), "color-set",
                       G_CALLBACK (cb_net_in_color_changed), app);
     title_text = g_strdup_printf(title_template, _("Receiving"));
     gsm_color_button_set_title(GSM_COLOR_BUTTON(color_picker), title_text);
@@ -279,7 +310,7 @@ create_sys_view (ProcmanApp *app, GtkBuilder * builder)
 
     color_picker = gsm_color_button_new (
         &net_graph->colors.at(1), GSMCP_TYPE_NETWORK_OUT);
-    g_signal_connect (G_OBJECT (color_picker), "color_set",
+    g_signal_connect (G_OBJECT (color_picker), "color-set",
                       G_CALLBACK (cb_net_out_color_changed), app);
     title_text = g_strdup_printf(title_template, _("Sending"));
     gsm_color_button_set_title(GSM_COLOR_BUTTON(color_picker), title_text);
@@ -294,26 +325,14 @@ create_sys_view (ProcmanApp *app, GtkBuilder * builder)
     gtk_grid_attach_next_to (GTK_GRID (table),  load_graph_get_labels(net_graph)->net_out_total, label, GTK_POS_RIGHT, 1, 1);
 
     app->net_graph = net_graph;
-    g_free(title_template);
+    g_free (title_template);
 
-    cpu_exp = GTK_WIDGET (gtk_builder_get_object (builder, "cpu_exp"));
-    mem_exp = GTK_WIDGET (gtk_builder_get_object (builder, "mem_exp"));
-    net_exp = GTK_WIDGET (gtk_builder_get_object (builder, "net_exp"));
-
-    g_object_bind_property(cpu_exp, "expanded", cpu_graph_box, "visible", G_BINDING_SYNC_CREATE);
-    g_object_bind_property(mem_exp, "expanded", mem_graph_box, "visible", G_BINDING_SYNC_CREATE);
-    g_object_bind_property(net_exp, "expanded", net_graph_box, "visible", G_BINDING_SYNC_CREATE);
-
-    g_settings_bind(app->settings, "show-cpu", cpu_exp, "expanded", G_SETTINGS_BIND_DEFAULT);
-    g_settings_bind(app->settings, "show-mem", mem_exp, "expanded", G_SETTINGS_BIND_DEFAULT);
-    g_settings_bind(app->settings, "show-network", net_exp, "expanded", G_SETTINGS_BIND_DEFAULT);
-    
 }
 
 static void
 on_activate_about (GSimpleAction *, GVariant *, gpointer data)
 {
-    ProcmanApp *app = (ProcmanApp *) data;
+    GsmApplication *app = (GsmApplication *) data;
 
     const gchar * const authors[] = {
         "Kevin Vandersloot",
@@ -324,6 +343,7 @@ on_activate_about (GSimpleAction *, GVariant *, gpointer data)
         "Karl Lattimer",
         "Chris Kühl",
         "Robert Roth",
+        "Stefano Facchini",
         NULL
     };
 
@@ -361,13 +381,13 @@ on_activate_about (GSimpleAction *, GVariant *, gpointer data)
 static void
 on_activate_refresh (GSimpleAction *, GVariant *, gpointer data)
 {
-    ProcmanApp *app = (ProcmanApp *) data;
-    proctable_update_all (app);
+    GsmApplication *app = (GsmApplication *) data;
+    proctable_update (app);
 }
 
 static void
-kill_process_with_confirmation (ProcmanApp *app, int signal) {
-    gboolean kill_dialog = g_settings_get_boolean (app->settings, "kill-dialog");
+kill_process_with_confirmation (GsmApplication *app, int signal) {
+    gboolean kill_dialog = g_settings_get_boolean (app->settings, GSM_SETTING_SHOW_KILL_DIALOG);
 
     if (kill_dialog)
         procdialog_create_kill_dialog (app, signal);
@@ -378,7 +398,7 @@ kill_process_with_confirmation (ProcmanApp *app, int signal) {
 static void
 on_activate_send_signal (GSimpleAction *, GVariant *parameter, gpointer data)
 {
-    ProcmanApp *app = (ProcmanApp *) data;
+    GsmApplication *app = (GsmApplication *) data;
 
     /* no confirmation */
     gint32 signal = g_variant_get_int32(parameter);
@@ -397,7 +417,7 @@ on_activate_send_signal (GSimpleAction *, GVariant *parameter, gpointer data)
 static void
 on_activate_memory_maps (GSimpleAction *, GVariant *, gpointer data)
 {
-    ProcmanApp *app = (ProcmanApp *) data;
+    GsmApplication *app = (GsmApplication *) data;
 
     create_memmaps_dialog (app);
 }
@@ -405,7 +425,7 @@ on_activate_memory_maps (GSimpleAction *, GVariant *, gpointer data)
 static void
 on_activate_open_files (GSimpleAction *, GVariant *, gpointer data)
 {
-    ProcmanApp *app = (ProcmanApp *) data;
+    GsmApplication *app = (GsmApplication *) data;
 
     create_openfiles_dialog (app);
 }
@@ -413,7 +433,7 @@ on_activate_open_files (GSimpleAction *, GVariant *, gpointer data)
 static void
 on_activate_process_properties (GSimpleAction *, GVariant *, gpointer data)
 {
-    ProcmanApp *app = (ProcmanApp *) data;
+    GsmApplication *app = (GsmApplication *) data;
 
     create_procproperties_dialog (app);
 }
@@ -435,45 +455,45 @@ on_activate_toggle (GSimpleAction *action, GVariant *parameter, gpointer data)
 static void
 change_show_page_state (GSimpleAction *action, GVariant *state, gpointer data)
 {
-    ProcmanApp *app = (ProcmanApp *) data;
+    GsmApplication *app = (GsmApplication *) data;
 
     g_simple_action_set_state (action, state);
-    g_settings_set_value (app->settings, "current-tab", state);
+    g_settings_set_value (app->settings, GSM_SETTING_CURRENT_TAB, state);
 }
 
 static void
 change_show_processes_state (GSimpleAction *action, GVariant *state, gpointer data)
 {
-    ProcmanApp *app = (ProcmanApp *) data;
+    GsmApplication *app = (GsmApplication *) data;
 
     g_simple_action_set_state (action, state);
-    g_settings_set_value (app->settings, "show-whose-processes", state);
+    g_settings_set_value (app->settings, GSM_SETTING_SHOW_WHOSE_PROCESSES, state);
 }
 
 static void
 change_show_dependencies_state (GSimpleAction *action, GVariant *state, gpointer data)
 {
-    ProcmanApp *app = (ProcmanApp *) data;
+    GsmApplication *app = (GsmApplication *) data;
 
     g_simple_action_set_state (action, state);
-    g_settings_set_value (app->settings, "show-dependencies", state);
+    g_settings_set_value (app->settings, GSM_SETTING_SHOW_DEPENDENCIES, state);
 }
 
 static void
 on_activate_priority (GSimpleAction *action, GVariant *parameter, gpointer data)
 {
-    ProcmanApp *app = (ProcmanApp *) data;
+    GsmApplication *app = (GsmApplication *) data;
 
     g_action_change_state (G_ACTION (action), parameter);
 
     const gint32 priority = g_variant_get_int32 (parameter);
     switch (priority) {
-	    case 32: 
-	        procdialog_create_renice_dialog (app);
-	        break;
-	    default:
-	        renice (app, priority);
-	        break;
+        case 32: 
+            procdialog_create_renice_dialog (app);
+            break;
+        default:
+            renice (app, priority);
+            break;
     }
 
 }
@@ -485,26 +505,31 @@ change_priority_state (GSimpleAction *action, GVariant *state, gpointer data)
 }
 
 void
-update_page_activities (ProcmanApp *app)
+update_page_activities (GsmApplication *app)
 {
     const char *current_page = gtk_stack_get_visible_child_name (GTK_STACK (app->stack));
 
     if (strcmp (current_page, "processes") == 0) {
+        GAction *search_action = g_action_map_lookup_action (G_ACTION_MAP (app->main_window),
+                                                             "search");
         proctable_thaw (app);
 
         gtk_widget_show (app->end_process_button);
-        gtk_widget_show (app->refresh_button);
-        gtk_widget_show (app->view_menu_button);
+        gtk_widget_show (app->search_button);
+        gtk_widget_show (app->process_menu_button);
 
         update_sensitivity (app);
 
-        gtk_widget_grab_focus (app->tree);
+        if (g_variant_get_boolean (g_action_get_state (search_action)))
+            gtk_widget_grab_focus (app->search_entry);
+        else
+            gtk_widget_grab_focus (app->tree);
     } else {
         proctable_freeze (app);
 
         gtk_widget_hide (app->end_process_button);
-        gtk_widget_hide (app->refresh_button);
-        gtk_widget_hide (app->view_menu_button);
+        gtk_widget_hide (app->search_button);
+        gtk_widget_hide (app->process_menu_button);
 
         update_sensitivity (app);
     }
@@ -520,23 +545,23 @@ update_page_activities (ProcmanApp *app)
     }
 
     if (strcmp (current_page, "disks") == 0) {
-		disks_update (app);
-		disks_thaw (app);
+        disks_update (app);
+        disks_thaw (app);
     } else {
-		disks_freeze (app);
+        disks_freeze (app);
     }
 }
 
 static void
 cb_change_current_page (GtkStack *stack, GParamSpec *pspec, gpointer data)
 {
-    update_page_activities ((ProcmanApp *)data);
+    update_page_activities ((GsmApplication *)data);
 }
 
 static gboolean
 cb_main_window_delete (GtkWidget *window, GdkEvent *event, gpointer data)
 {
-    ProcmanApp *app = (ProcmanApp *) data;
+    GsmApplication *app = (GsmApplication *) data;
 
     app->shutdown ();
 
@@ -544,13 +569,14 @@ cb_main_window_delete (GtkWidget *window, GdkEvent *event, gpointer data)
 }
 
 void
-create_main_window (ProcmanApp *app)
+create_main_window (GsmApplication *app)
 {
-    gint width, height, xpos, ypos;
     GtkWidget *main_window;
     GtkWidget *stack;
-    GtkWidget *view_menu_button;
-    GMenuModel *view_menu_model;
+    GtkWidget *process_menu_button;
+    GMenuModel *process_menu_model;
+
+    int width, height, xpos, ypos;
 
     GtkBuilder *builder = gtk_builder_new();
     gtk_builder_add_from_resource (builder, "/org/gnome/gnome-system-monitor/data/interface.ui", NULL);
@@ -561,15 +587,27 @@ create_main_window (ProcmanApp *app)
     gtk_widget_set_name (main_window, "gnome-system-monitor");
     app->main_window = main_window;
 
-    app->view_menu_button = view_menu_button = GTK_WIDGET (gtk_builder_get_object (builder, "viewmenubutton"));
-    view_menu_model = G_MENU_MODEL (gtk_builder_get_object (builder, "view-menu"));
-    gtk_menu_button_set_menu_model (GTK_MENU_BUTTON (view_menu_button), view_menu_model);
+    g_settings_get (app->settings, GSM_SETTING_WINDOW_STATE, "(iiii)",
+                    &width, &height, &xpos, &ypos);
+    width = CLAMP (width, 50, gdk_screen_width ());
+    height = CLAMP (height, 50, gdk_screen_height ());
+
+    gtk_window_set_default_size (GTK_WINDOW (main_window), width, height);
+    gtk_window_move (GTK_WINDOW (main_window), xpos, ypos);
+    if (g_settings_get_boolean (app->settings, GSM_SETTING_MAXIMIZED))
+        gtk_window_maximize (GTK_WINDOW (main_window));
+
+    app->process_menu_button = process_menu_button = GTK_WIDGET (gtk_builder_get_object (builder, "process_menu_button"));
+    process_menu_model = G_MENU_MODEL (gtk_builder_get_object (builder, "process-window-menu"));
+    gtk_menu_button_set_menu_model (GTK_MENU_BUTTON (process_menu_button), process_menu_model);
 
-    app->refresh_button = GTK_WIDGET (gtk_builder_get_object (builder, "refresh_button"));
     app->end_process_button = GTK_WIDGET (gtk_builder_get_object (builder, "end_process_button"));
 
+    app->search_button = GTK_WIDGET (gtk_builder_get_object (builder, "search_button"));
+
     GActionEntry win_action_entries[] = {
         { "about", on_activate_about, NULL, NULL, NULL },
+        { "search", on_activate_toggle, NULL, "false", NULL },
         { "send-signal-stop", on_activate_send_signal, "i", NULL, NULL },
         { "send-signal-cont", on_activate_send_signal, "i", NULL, NULL },
         { "send-signal-end", on_activate_send_signal, "i", NULL, NULL },
@@ -596,16 +634,6 @@ create_main_window (ProcmanApp *app)
     if (visual)
         gtk_widget_set_visual(main_window, visual);
 
-    width = app->config.width;
-    height = app->config.height;
-    xpos = app->config.xpos;
-    ypos = app->config.ypos;
-    gtk_window_set_default_size (GTK_WINDOW (main_window), width, height);
-    gtk_window_move(GTK_WINDOW (main_window), xpos, ypos);
-    if (app->config.maximized) {
-        gtk_window_maximize(GTK_WINDOW(main_window));
-    }
-
     /* create the main stack */
     app->stack = stack = GTK_WIDGET (gtk_builder_get_object (builder, "stack"));
 
@@ -615,7 +643,7 @@ create_main_window (ProcmanApp *app)
     
     create_disk_view (app, builder);
 
-    g_settings_bind (app->settings, "current-tab", stack, "visible-child-name", G_SETTINGS_BIND_DEFAULT);
+    g_settings_bind (app->settings, GSM_SETTING_CURRENT_TAB, stack, "visible-child-name", G_SETTINGS_BIND_DEFAULT);
 
     g_signal_connect (G_OBJECT (stack), "notify::visible-child",
                       G_CALLBACK (cb_change_current_page), app);
@@ -628,15 +656,15 @@ create_main_window (ProcmanApp *app)
     action = g_action_map_lookup_action (G_ACTION_MAP (main_window),
                                          "show-dependencies");
     g_action_change_state (action,
-                           g_settings_get_value (app->settings, "show-dependencies"));
+                           g_settings_get_value (app->settings, GSM_SETTING_SHOW_DEPENDENCIES));
 
 
     action = g_action_map_lookup_action (G_ACTION_MAP (main_window),
                                          "show-whose-processes");
     g_action_change_state (action,
-                           g_settings_get_value (app->settings, "show-whose-processes"));
+                           g_settings_get_value (app->settings, GSM_SETTING_SHOW_WHOSE_PROCESSES));
 
-    gtk_widget_show_all(main_window);
+    gtk_widget_show (main_window);
     
     update_page_activities (app);
 
@@ -644,26 +672,7 @@ create_main_window (ProcmanApp *app)
 }
 
 void
-do_popup_menu (ProcmanApp *app, GdkEventButton *event)
-{
-    guint button;
-    guint32 event_time;
-
-    if (event) {
-        button = event->button;
-        event_time = event->time;
-    }
-    else {
-        button = 0;
-        event_time = gtk_get_current_event_time ();
-    }
-
-    gtk_menu_popup (GTK_MENU (app->popup_menu), NULL, NULL,
-                    NULL, NULL, button, event_time);
-}
-
-void
-update_sensitivity(ProcmanApp *app)
+update_sensitivity(GsmApplication *app)
 {
     const char * const selected_actions[] = { "send-signal-stop",
                                               "send-signal-cont",
@@ -675,6 +684,7 @@ update_sensitivity(ProcmanApp *app)
                                               "process-properties" };
 
     const char * const processes_actions[] = { "refresh",
+                                               "search",
                                                "show-whose-processes",
                                                "show-dependencies" };
 
@@ -683,17 +693,22 @@ update_sensitivity(ProcmanApp *app)
     GAction *action;
 
     processes_sensitivity = (strcmp (gtk_stack_get_visible_child_name (GTK_STACK (app->stack)), "processes") == 0);
-    selected_sensitivity = (processes_sensitivity && app->selected_process != NULL);
+    selected_sensitivity = gtk_tree_selection_count_selected_rows (app->selection) > 0;
 
-    for (i = 0; i != G_N_ELEMENTS(processes_actions); ++i) {
+    for (i = 0; i != G_N_ELEMENTS (processes_actions); ++i) {
         action = g_action_map_lookup_action (G_ACTION_MAP (app->main_window),
-                                              processes_actions[i]);
-        g_simple_action_set_enabled (G_SIMPLE_ACTION (action), processes_sensitivity);
+                                             processes_actions[i]);
+        g_simple_action_set_enabled (G_SIMPLE_ACTION (action),
+                                     processes_sensitivity);
     }
 
-    for (i = 0; i != G_N_ELEMENTS(selected_actions); ++i) {
+    for (i = 0; i != G_N_ELEMENTS (selected_actions); ++i) {
         action = g_action_map_lookup_action (G_ACTION_MAP (app->main_window),
                                              selected_actions[i]);
-        g_simple_action_set_enabled (G_SIMPLE_ACTION (action), selected_sensitivity);
+        g_simple_action_set_enabled (G_SIMPLE_ACTION (action),
+                                     processes_sensitivity & selected_sensitivity);
     }
+
+    gtk_revealer_set_reveal_child (GTK_REVEALER (app->proc_toolbar_revealer),
+                                   selected_sensitivity);
 }
